#!/usr/bin/perl
use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);
use Math::Approx;
use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;
use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;
use PDL::Core;
use PDL::Graphics::LUT;
use Carp;
use PDL::Image2D;

#$ENV{PGPLOT_FOREGROUND} = "black";
#$ENV{PGPLOT_BACKGROUND} = "white";

$ENV{PGPLOT_ENVOPT} = "V";

require "/home/sanchez/sda1/perl/MY/my.pl";

if ($#ARGV<13) {
    print "USE: maps_orion.pl map.fits min max bright contrast Label factor dev VAL_TO_WHITE MAP_CONT NLEVELS MIN STEP NWHITE[49] [TABLE REVERSE] [ROT]\n";
    exit;
}
$mapfile=$ARGV[0];
$min=$ARGV[1];
$max=$ARGV[2];
$bright=$ARGV[3];
$contrast=$ARGV[4];
$label=$ARGV[5];
$factor=$ARGV[6];
$dev=$ARGV[7];
$vw=$ARGV[8];
$map_cont=$ARGV[9];
$nlevels=$ARGV[10];
$min_cont=$ARGV[11];
$steps=$ARGV[12];
$NW=$ARGV[13];
if ($dev !~ "TPNG") { 
    $ENV{PGPLOT_FOREGROUND} = "black";
    $ENV{PGPLOT_BACKGROUND} = "white";
} else {
    $ENV{PGPLOT_FOREGROUND} = "white";
    $ENV{PGPLOT_BACKGROUND} = "white";
}

$table="idl5"; $reverse=1; 
if ($#ARGV==15){
    $table=$ARGV[14];
    $reverse=$ARGV[15];
}
$rot=0;
if ($#ARGV==16){
    $table=$ARGV[14];
    $reverse=$ARGV[15];
    $rot=$ARGV[16];
}




#$table="smooth2"; $reverse=0; 
#$table="real"; $reverse=0; 
#$bright=0.7; 
#$contra=0.5; 
$color_cont=0;

#$pdl_mask=rfits("M74_mos.mask.fits");
#$pdl_mask=rfits("mask_sc.fits");
$pdl_map_tmp=rfits($mapfile);
$pdl_cont_tmp=rfits($map_cont);

$is_zero=sum(abs($pdl_map_tmp));


if ($rot==0) {
    $pdl_map=$pdl_map_tmp;
    $pdl_cont=$pdl_cont_tmp;
} else {
#    $pdl_map=$pdl_map_tmp->rot2d($rot,0,1);
#    $pdl_cont=$pdl_cont_tmp->rot2d($rot,0,1);
    $pdl_map=transpose($pdl_map_tmp);
    $pdl_cont=transpose($pdl_cont_tmp);
}


#$crval1=$pdl_map->hdr->{CRVAL1};
#$cdelt1=$pdl_map->hdr->{CDELT1};
#$crval2=$pdl_map->hdr->{CRVAL2};
#$cdelt2=$pdl_map->hdr->{CDELT2};
#print "$crval1 $cdelt1 $crval2 $cdelt2\n";
($nx,$ny)=$pdl_map->dims;
$h=$pdl_map->gethdr;
$crval1=-138.839996337891;
$cdelt1=1;
$crval2=-210.726806640625;
$cdelt2=1;


$pdl_map=$pdl_map*$factor;


$NNx=$nx-1;
$NNy=$ny-1;
$rpdl_map=$pdl_map->slice("$NNx:0,$NNy:0");
@map=list($rpdl_map);
$rpdl_cont=$pdl_cont->slice("$NNx:0,$NNy:0");
@cont=list($rpdl_cont);
$crval1=$h->{CRVAL1};
$crval2=$h->{CRVAL2};
$cdelt1=$h->{CD1_1};
$cdelt2=$h->{CD2_2};
print "$crval1 $crval2 | '$cdelt1' '$cdelt2'\n";
$crval2=$crval2-$cdelt2*$h->{CRPIX2};
$crval1=3600*($crval1-$cdelt1*$h->{CRPIX1})/15;
$cdelt1=$cdelt1*(3600/15)/cos($crval2/180*3.1416);
$cdelt2=3600*$cdelt2;
$crval2=3600*$crval2;
print "$crval1 $cdelt1\n";

$x_max=$crval1;
$x_min=$crval1+$cdelt1*$nx;
$y_min=$crval2;
$y_max=$crval2+$cdelt2*$ny;
for ($j=0;$j<$ny;$j++) {
    for ($i=0;$i<$nx;$i++) {
#	if ($map[$i+$j*$nx]<0) {
#	    $map[$i+$j*$nx]=0;
#	}
	if ($map[$i+$j*$nx]>1e12) {
	    $map[$i+$j*$nx]=0;
	}
	if ($map[$i+$j*$nx] eq "nan") {
	    $map[$i+$j*$nx]=0;
#
	}
	if ($map[$i+$j*$nx]!=(1*$map[$i+$j*$nx])) {
#	    print "$i $j $map[$i+$j*$nx]\n";
	    $map[$i+$j*$nx]=0;
	}
	if ($map[$i+$j*$nx]==$vw) {
	    $map[$i+$j*$nx]=-1e12;
	}
#	print "$map[$i+$j*$nx]\n";

    }
}


#@tr=($crval1,$cdelt1,0,$crval2,0,$cdelt2);
#@tr=($crval1,$cdelt1,0,$crval2,0,$cdelt2);
@tr=($x_min,(-1)*$cdelt1,0,$y_max,0,(-1)*$cdelt2);



#@tr=(0,1,0,0,0,1);
#$x_min=0;
#$x_max=$nx;
#$y_min=0;
#$y_max=$ny;

pgbeg(0,$dev,1,1);
pgscf(2.0);
pgsch(1.2);

#open(DATA);
while($cmap=<DATA>) {
    chop($cmap);
    @data=split(" ",$cmap);
    $nc=$data[0];
    $b[238-$nc]=$data[1];
    $g[238-$nc]=$data[2];
    $r[238-$nc]=$data[3];
    $l[$nc]=$nc/238;
}
#close(DATA);
$bright=1;
$NW=0;

for ($j=0;$j<$NW;$j++) {
    $r[$j]=1;
    $g[$j]=1;
    $b[$j]=1;
}
pgctab(\@l,\@r,\@g,\@b,$nc,$bright,$contrast);

pgsvp(0.15,0.8,0.1,0.9);
pgswin($x_max,$x_min,$y_min,$y_max);
pgtbox("YZN",0,0,"YZNV",0,0);

#pgenv($x_min,$x_max,$y_min,$y_max,1,0);
#pgenv($x_min,$x_max,$y_min,$y_max,1,0);
#pgpap(6.0,1.0);

#pgsch(0.7);           # Set character height
#pgsch(1.2);

if ($dev !~ "TPNG") { 
  #  pglabel("RA","DEC","");
    pgmtxt("L",3,0.45,0.5,"Declination");
    pgmtxt("BV",3,0.5,0.5,"Right Ascesion");
}
#pglabel("\\gD RA (arcsec)","\\gD DEC (arcsec)","");
pgimag(\@map,$nx,$ny,1,$nx,1,$ny,$min,$max,\@tr);

pgsci(0);
pgsch(1);
#pgsch(1.1);
@tmp;
$nt=0;
for ($j=0;$j<$ny;$j++) {
    for ($i=0;$i<$nx;$i++) {
	$val=$map[$i+$j*$nx];
#	if ($map[$i+$j*$nx]<=0) {
	if ($val<=0) {
	    $X=$crval1+$cdelt1*$i+1;
	    $Y=$crval2+$cdelt2*$j+1;
	#    print "$i,$j $X,$Y MAP=$map[$i+$j*$nx]\n";
#	    pgpoint(1,[$X],[$Y],16);
	} else {
	    if ($val>6) {
		$tmp[$nt]=$val;
#		print "$tmp[$nt] $nt\n";
		$nt++;
	    }
	}
    }
}
$mean=mean(@tmp);
$sigma=sigma(@tmp);
$sum=$nt*$mean;
print "FLUX=$sum\n";
print "MEAN=$mean+-$sigma\n";


pgsch(3);
pgsci(3);
pgslw(3);
#pgpoint1,[3.65],[-3.13],3);
#pgpoint(1,0,0,3);
pgsch(1.2);
pgslw(1);

pgsci(1);
pgsch(1.3);
if ($dev !~ "TPNG") { 
    pgsci(1);
    pgsch(1.3);
    if ($rot==0) {
	pgptxt(-15,28,0,0.5,$label);
    } else {
	pgptxt(-15,24,0,0.5,$label);
    }
#pgptxt(-13,28,0,0.5,$label);
}

#pgptxt(-13,28,0,0.5,$label);
pgsci(1);
pgsch(1);

pgsci(1);
pgslw(3);
#$nlevels=10;
#$min_cont=$ARGV[11];
#$steps=$ARGV[12];
for ($i=0;$i<$nlevels;$i++) {    
    $levels[$i]=$min_cont+$steps*($i**1.5);
#    print "$i $levels[$i]\n";
}

pgsci(1);
pgslw(3);
if ($dev !~ "TPNG") { 
    pgsci(1);
    pgslw(3);
} else { 
    if ($reverse==0) {
	pgsci(4);
    } else {
	pgsci(4);
    }
    pgslw(8);
}



if ($is_zero!=0) {
    pgcont(\@cont,$nx,$ny,1,$nx,1,$ny,\@levels,$nlevels,\@tr);
}
pgsci(1);
pgslw(1);

if ($dev !~ "TPNG") { 
#pgptxt(-85,160,0,0,$label);
#pgptxt(-110,160,0,0,$label);
pgwedg("RI",0.0,4.5,$min,$max,"");
#pgwedg("RI",0,1.0,1,500,"");
pgbox("SBC",0,0,"SBC",0,0);
}




pgclos();
pgend();



exit;


__DATA__
    0  0.0  0.0  0.0
    1  0.16836600832008092  0.0  0.0
    2  0.22117646780668571  0.0  0.0
    3  0.26196079315946302  0.0  0.0
    4  0.30509804322439249  0.0  0.0
    5  0.34640523078881835  0.0  0.0
    6  0.3890196093040354  0.0  0.0
    7  0.4329411779314864  0.0  0.0
    8  0.45490196347236633  0.0054901937724045119  0.0
    9  0.45490196347236633  0.016470584016950333  0.0015686260302567555
    10  0.45490196347236633  0.02614378918424869  0.005228757183828231
    11  0.45490196347236633  0.034509802358407601  0.0078431377187371254
    12  0.45568627629209968  0.044705881695537006  0.0086274505384704644
    13  0.45882353186607361  0.052549018775130581  0.012287581381600904
    14  0.45882353186607361  0.070588238128260067  0.019607843626655783
    15  0.45882353186607361  0.081568620345180351  0.019607843831181526
    16  0.45882353186607361  0.089150322443909108  0.023006533193919394
    17  0.46196078355405845  0.099607835569802447  0.026666663912873641
    18  0.46274510025978088  0.10771241371343339  0.030326795997766332
    19  0.46274510025978088  0.11764705148982067  0.031372550874948502
    20  0.46274510025978088  0.12627450945330151  0.033725489538674262
    21  0.46274510025978088  0.13568627486041948  0.03738562025762851
    22  0.46457516357400058  0.14483660370306251  0.041045750976582758
    23  0.46666666865348816  0.15372549078043771  0.043137256056070328
    24  0.46666666865348816  0.1633986949725868  0.044444444020783978
    25  0.46666666865348816  0.17071895641049528  0.048104574739738226
    26  0.46666666865348816  0.17882353046361138  0.051764705458692474
    27  0.47058823704719543  0.19660130911792806  0.058823529630899429
    28  0.47058823704719543  0.20392157055583654  0.058823529630899429
    29  0.47058823704719543  0.2149019623036478  0.062483663417739622
    30  0.47058823704719543  0.22248364977199836  0.066143788358365513
    31  0.47058823704719543  0.22980390935551886  0.069803918150125766
    32  0.47346404737896391  0.23712417079342735  0.070588238537311554
    33  0.47450980544090271  0.24705881482243491  0.073202611194326986
    34  0.47450980544090271  0.25568627696411284  0.076862741913281241
    35  0.47450980544090271  0.26300653840202132  0.080522872632235482
    36  0.47450980544090271  0.2703267998399298  0.084183003351189736
    37  0.47607842739890605  0.27764706127783834  0.086274512112140656
    38  0.47843137383460999  0.28496732271574682  0.087581696395390957
    39  0.47843137383460999  0.29333333076215257  0.091241827114345198
    40  0.47843137383460999  0.31084967542317959  0.098562088552253693
    41  0.47869280741884818  0.31816993686108808  0.10196078568696976
    42  0.48235293813780244  0.32549019829899656  0.10196078568696976
    43  0.48235294222831726  0.33281045973690504  0.10562091231540917
    44  0.48235294222831726  0.34013072117481358  0.10928104303436342
    45  0.48235294222831726  0.34745098261272206  0.11294117375331766
    46  0.48235294222831726  0.35477124405063054  0.11660130447227192
    47  0.48496731815774458  0.36209150548853908  0.12026143519122616
    48  0.48627451062202454  0.36941176692644756  0.12156862765550613
    49  0.48627451062202454  0.37673202836435604  0.12366013220906549
    50  0.48627451062202454  0.38405228980226452  0.12732026640496222
    51  0.48627451062202454  0.38980392600975783  0.13098039712391649
    52  0.48758169817768671  0.39477124428437427  0.13464052784287073
    53  0.49019607901573181  0.40941176716019123  0.14117647707462311
    54  0.49019607901573181  0.41673202859809977  0.1416993516060262
    55  0.49019607901573181  0.42379085317935816  0.14535948232498044
    56  0.49019607901573181  0.42745098389831243  0.14901961304393468
    57  0.49385620891658311  0.43477124451811799  0.15267974376288895
    58  0.49411764740943909  0.44209150595602648  0.15633987448184319
    59  0.49411764740943909  0.44941176739393496  0.16000000520079743
    60  0.49411764740943909  0.45385620175627023  0.16078431904315948
    61  0.49411764740943909  0.4601306940605438  0.16339868433724822
    62  0.49647057502877479  0.46745095549845228  0.16705881505620246
    63  0.49803921580314636  0.47267972699177813  0.1707189457751567
    64  0.49803921580314636  0.47816990998056202  0.17437907649411097
    65  0.49803921580314636  0.48549017141847051  0.17803920721306521
    66  0.49803921580314636  0.49150325593605543  0.18169933793201945
    67  0.50039220489716851  0.50196081399917603  0.18823529779911041
    68  0.4935947835133746  0.50248367789524051  0.18823529779911041
    69  0.48993465279442033  0.5058823823928833  0.18849672402042189
    70  0.48627452207546606  0.5058823823928833  0.19215685473937613
    71  0.48261439135651185  0.50954250165839599  0.19215686619281769
    72  0.47895426063755758  0.5132026323773502  0.19555554778357737
    73  0.4721568860025967  0.51372551918029785  0.19607843458652496
    74  0.46771243080594183  0.51660132542155146  0.19895424082777857
    75  0.46405230008698756  0.52026145614050567  0.20000000298023224
    76  0.46039216936803329  0.5215686559677124  0.20235293387197981
    77  0.45464054543208454  0.52366014918470694  0.20392157137393951
    78  0.44915033953641753  0.52549022436141968  0.20575162691618104
    79  0.44549020881746332  0.5270588422289082  0.20784313976764679
    80  0.43424837898584751  0.53333336114883423  0.21176470816135406
    81  0.43058824826689329  0.53411766599206367  0.21254901300458348
    82  0.42692811754793902  0.53777779671101789  0.21568627655506134
    83  0.42300655733526143  0.54117649793624878  0.21594770604878469
    84  0.41568629589735295  0.54117649793624878  0.21960783676773893
    85  0.412026156997369  0.54483662047417336  0.21960784494876862
    86  0.40836602627841478  0.54849675119312757  0.22300652981194016
    87  0.401568648371042  0.54901963472366333  0.22352941334247589
    88  0.39712419644679897  0.55189544423732884  0.2264052228561414
    89  0.39346406572784476  0.55294120311737061  0.22745098173618317
    90  0.38745100084473105  0.5552941372815301  0.2298039159003426
    91  0.38222223589622895  0.55895426800048431  0.23137255012989044
    92  0.37856210517727473  0.56078433990478516  0.23320260894454384
    93  0.36732027534565892  0.56601309176363979  0.23921568691730499
    94  0.36366014462670471  0.56862747669219971  0.24026143270769929
    95  0.35921570579210915  0.56941178480784094  0.24313725531101227
    96  0.35241831479508895  0.57254904508590698  0.24366012575190052
    97  0.34875818407613468  0.5728104778520422  0.24705882370471954
    98  0.34509805335718041  0.57647060857099641  0.24705882370471954
    99  0.33777779682788972  0.58013073928995074  0.2507189634227886
    100  0.33385622352556465  0.58039218187332153  0.25098040699958801
    101  0.33019609280661039  0.58352943233415189  0.25411765746041842
    102  0.32366014930157877  0.58718956305310621  0.25490197539329529
    103  0.31895426297499463  0.58823531866073608  0.25751635050461963
    104  0.31529413225604036  0.59058825609730736  0.2611764812235739
    105  0.30954250177526788  0.59215688705444336  0.26274511218070984
    106  0.30039217170547039  0.59764707986046284  0.26666668057441711
    107  0.29542485424895693  0.60000002384185791  0.26797386731197631
    108  0.28915034187385458  0.6010457729046641  0.27058824896812439
    109  0.28470589976684718  0.60470590362361831  0.27137256035617752
    110  0.27790851204223882  0.60784316062927246  0.27503269107513179
    111  0.27477125340037878  0.60810459666781957  0.27895425783263311
    112  0.27843138411933299  0.61176472738677379  0.28627451927054159
    113  0.28209151483828726  0.61176472902297974  0.29725490979119845
    114  0.28235295414924622  0.61516342043097505  0.30483661054006589
    115  0.28549020788248847  0.61882355114992926  0.31529412571121662
    116  0.28627452254295349  0.61960786581039429  0.32339870180959018
    117  0.28888890092668967  0.62222224419413052  0.33333334163123485
    118  0.29254903164564394  0.62588237491308474  0.34196079307911442
    119  0.2959477117093694  0.62928105497681019  0.36235291076668846
    120  0.29803922772407532  0.63137257099151611  0.37176468080142128
    121  0.29934640846227467  0.63267975172971547  0.38039213781281883
    122  0.30196079611778259  0.63633987132226821  0.39032676465343225
    123  0.30274510521517994  0.63921570777893066  0.3984313648589492
    124  0.30640521739027193  0.63973856065771273  0.40888884850544316
    125  0.30980393290519714  0.64339871362923329  0.41647059190507957
    126  0.30980393290519714  0.64705881838723589  0.4274509061790876
    127  0.31346406340599059  0.64705884456634521  0.4347712457180023
    128  0.31372550129890442  0.65045751514014116  0.44549013322521797
    129  0.31686275274143516  0.65098041296005249  0.45647058101254401
    130  0.32052286862560569  0.65385621189304644  0.46457512973206161
    131  0.32156863808631897  0.65751635373802775  0.47189541342202418
    132  0.32156863808631897  0.65882354974746704  0.47686273273309471
    133  0.32549020648002625  0.66065360539885709  0.48993462192480453
    134  0.32549020648002625  0.66274511814117432  0.49725488786575373
    135  0.32679739049061424  0.66405230215176236  0.50326799800976407
    136  0.32941177487373352  0.66666668653488159  0.50797385281184304
    137  0.32941177487373352  0.66745099890466764  0.51529414391986983
    138  0.32993463102495069  0.67058825492858887  0.52261436827014673
    139  0.3333333432674408  0.67084969565757291  0.52967323348411099
    140  0.3333333432674408  0.67450980041557551  0.5333333382421136
    141  0.3333333432674408  0.67450982332229614  0.54065362869529132
    142  0.33673201711362544  0.67450982332229614  0.54797384562861806
    143  0.33725491166114807  0.67764707803726199  0.55215688943862917
    144  0.33725491166114807  0.67843139171600342  0.55869280752813588
    145  0.33986928731785115  0.6810457673727065  0.56601309121819132
    146  0.34117648005485535  0.68444445670815113  0.58065360667649502
    147  0.34300653897863398  0.68627452850341797  0.58614380919080744
    148  0.34509804844856262  0.68627452850341797  0.59137255374109143
    149  0.34509804844856262  0.6875817157863946  0.59869283001410389
    150  0.34614378693047759  0.69019609689712524  0.60601306919939513
    151  0.3490196168422699  0.69098041253929987  0.61254904466515436
    152  0.3490196168422699  0.69411766529083252  0.61673201626399143
    153  0.34928106084364252  0.69437910929220514  0.62405232220684681
    154  0.35294116560164523  0.69803921405020786  0.63137253172285224
    155  0.35294118523597717  0.69803923368453979  0.6386928450828433
    156  0.35294118523597717  0.70143791080311313  0.64601306201607733
    157  0.35607844322337806  0.70196080207824707  0.65019610255374616
    158  0.35686275362968445  0.70196080207824707  0.65673202391559515
    159  0.35921568746665378  0.70588237047195435  0.67137255420882025
    160  0.36078432202339172  0.70797387034282977  0.6786928230640471
    161  0.36078432202339172  0.70980393886566162  0.68418302230587802
    162  0.3623529428360045  0.7113725596782744  0.68941177012864352
    163  0.364705890417099  0.7137255072593689  0.69673204640156317
    164  0.364705890417099  0.71477124901371891  0.7040522855869471
    165  0.36549020933166232  0.71764707565307617  0.71137257669478826
    166  0.36862745881080627  0.7171242129569888  0.71764707565307617
    167  0.36862745881080627  0.71346405998560758  0.71790852292683749
    168  0.36862745881080627  0.70980395522760487  0.7215686276848402
    169  0.37228759912890497  0.70980393886566162  0.72156864404678345
    170  0.37254902720451355  0.70640525847469959  0.72496732443774548
    171  0.37254902720451355  0.70274510921211841  0.72549021244049072
    172  0.37647059559822083  0.69542485148296651  0.729411780834198
    173  0.37882353270757879  0.6917647281814745  0.73176471794355602
    174  0.3803921639919281  0.68810459375381472  0.73333334922790527
    175  0.3803921639919281  0.68444446303486195  0.73333334922790527
    176  0.38196078807697592  0.68078433602466282  0.73490197331295315
    177  0.38431373238563538  0.6771241978882494  0.73725491762161255
    178  0.38431373238563538  0.67346407829551103  0.73830066264839767
    179  0.38509805457258733  0.66980393274163685  0.74117648601531982
    180  0.38823530077934265  0.66614382056635923  0.74169935198384218
    181  0.38823530077934265  0.66248366759502431  0.7450980544090271
    182  0.38823530077934265  0.6588235628370217  0.7450980544090271
    183  0.39189544436987644  0.65516340615693325  0.7450980544090271
    184  0.39215686917304993  0.65150329769040916  0.74849673807237771
    185  0.39215686917304993  0.64784314842778146  0.74901962280273438
    186  0.3960784375667572  0.64052289069862955  0.75294119119644165
    187  0.40313725871199713  0.63921570777893066  0.75529413157818825
    188  0.41202615557937028  0.64130721419467629  0.76104577242164018
    189  0.41934641701718284  0.64313727617263794  0.76653596511850808
    190  0.42823529839515684  0.64470590353012081  0.77019609212875362
    191  0.43790851570411521  0.64705884456634521  0.77385623026512063
    192  0.4462745031889051  0.64810459286556543  0.77751634985790552
    193  0.45647061439104758  0.65176473841939309  0.78196082087107388
    194  0.46431370798265331  0.65490198135375977  0.78875817598076459
    195  0.47503271307797995  0.65516343517229836  0.792418328952053
    196  0.48235292259398532  0.65882353993030107  0.79607843371005571
    197  0.49333338281722383  0.66248369661043593  0.79973859039019057
    198  0.50431373404529323  0.66614380507691362  0.80339869885666826
    199  0.52235296506510975  0.67346407022352617  0.81464053239698808
    200  0.53333339060074991  0.67712421206873952  0.81830067424220143
    201  0.5419608168508121  0.68078433537013872  0.82196079754360063
    202  0.55137259539449823  0.68444446979789142  0.82562093197135333
    203  0.56235298755107777  0.68810460051675126  0.83111113470354647
    204  0.57333336858195372  0.69176472752704321  0.8368627580942124
    205  0.58300658261838412  0.69542486566336381  0.840522896230533
    206  0.59137257337570193  0.69908498525619511  0.8441830158233643
    207  0.60235301003704578  0.70352945954170565  0.84784316137714555
    208  0.61333334656315741  0.71032681137905418  0.85202614606590865
    209  0.62431380547688342  0.71424842144122347  0.85908499491746537
    210  0.63529411975089145  0.72156863095722878  0.86274509967546809
    211  0.64627458979143526  0.72522879418218744  0.86640525635564936
    212  0.66509809190174951  0.73647062030504651  0.87372551408480115
    213  0.67529414223685946  0.74300655475626443  0.88026144853601906
    214  0.68627456777263895  0.75032683844678416  0.88496734020766021
    215  0.69725493767669711  0.75764708504948952  0.888627463509013
    216  0.70823534096009444  0.76496735390508774  0.89228759793681212
    217  0.71921573311653475  0.77228761534271462  0.89594772865562555
    218  0.73176474804990299  0.77960786936339144  0.90117648956831697
    219  0.74509809695007967  0.78692814563593971  0.90718956219594538
    220  0.75607845572871268  0.79424838482169513  0.91084968178882297
    221  0.76705889238991731  0.80156867592916481  0.91450982734255792
    222  0.77803922891616817  0.80888890027999882  0.91816993951797488
    223  0.78901968782975496  0.81647066658570588  0.92209155285248634
    224  0.80000000210376299  0.82745098085971391  0.92941176236849177
    225  0.82535949102411721  0.8420915144253277  0.9367320307874929
    226  0.83686281372560878  0.85254908730043788  0.94039218005025993
    227  0.84784316137714555  0.86352943495197465  0.94405229593410545
    228  0.85882358691306426  0.87189546519657979  0.9503268330707253
    229  0.87215690701589577  0.88156866199810502  0.95529412947442527
    230  0.88470592849422713  0.89254906528164168  0.95895426390227079
    231  0.89568632065052811  0.90352945743794266  0.96261439462103782
    232  0.90823533885647079  0.91450983846909717  0.96627452163142269
    233  0.92156868448407303  0.92549025287778031  0.97124186668495904
    234  0.93359480137937079  0.93647061165655265  0.97751634775428176
    235  0.94745104831761795  0.94745104831761795  0.98117649330797019
    236  0.95895426390227079  0.95895426390227079  0.98483660548343366
    237  0.97333341215116287  0.97333341215116287  0.98849675845458274
    238  0.9843137264251709  0.9843137264251709  0.99215686321258545
